from time import localtime

def lcl_time(nb_month=1):
	tps = localtime()
	tm_year, tm_mon, tm_mday = tps.tm_year, tps.tm_mon, tps.tm_mday
	if tm_mon <= nb_month:
		if nb_month >= 12:
			tm_year -= int(nb_month/12)
			tm_mon -= (nb_month % 12)
		else:
			tm_mon = 12 + (tm_mon - nb_month)
			tm_year -= 1
	else:
		tm_mon -= nb_month
	tm_mday = str(tm_mday)
	tm_mon = str(tm_mon)
	tm_year = str(tm_year)
	if len(tm_mday) == 1:
		tm_mday = '0' + tm_mday
	elif len(tm_mon) == 1:
		tm_mon = '0' + tm_mon
	return int(tm_year + tm_mon + tm_mday + '000000')

	



import time
import os
import sys
from .src.analyzer import (
	html_init,
	init,
	html_end,
	raw_end,
	list_end)
from .src.swy import (
	swy,
	generate_swy)
from .src.time import lcl_time

def check_id(id):
	if id[:2] == 'UC' or id[:2] == 'PL':
		return True
	else:
		return False

def main():
	url_data = []
	analyze = False
	analyze_only_one = False
	mode = 'html'
	count = 7
	sub_file = 'subscription_manager'
	#path
	if os.name == 'nt':
		path = ''
	else:
		system = os.uname().sysname
		if system == 'Linux':
			path = os.environ['HOME'] + '/.cache/youtube_sm/'
			try:
				os.makedirs(path + 'data/')
			except:
				pass
	#commands
	del sys.argv[0]
	if sys.argv==[]:
		passe = time.time()
		url_data = swy(path=path)
		html_init(path)
		nb_new = init(url_data, lcl_time(), path=path)
		html_end(path=path)
		open(path + 'log', 'a').write(str(time.time() - passe) + '\t' + str(nb_new) + '\t' + time.strftime("%H%M") + '\n')
	elif sys.argv == ['-h']:
		print("""Usage: youtube-sm [OPTIONS]

Options:
	-h			Print this help text and exit
	-n  [file]		To use an other xml file for yours subscriptions
	-m  [mode] 		The type of file do you want (html, raw, list)
	-t  [nb of days]	Numbers of days of subscriptions do you want in your file
	-d			Show the dead channels + those who posted no videos
	-o  [nb of months]	Show the channels who didn't post videos in nb of months + dead channels
	-a  [id]		To append a channel or a playlist at the end of sub.swy
	-af [file]		To append a file with list of channel or a playlist in sub.swy
	-ax [file]		To append a xml file in sub.swy
	-l  [id]		If you want to analyze only one channel or playlist
	-r			To remove the cache before analyze
	-s  [id/all]		To have the stat of the channel(s)
""", end='')
	else:
		for arg in range(len(sys.argv)):
			if sys.argv[arg] == '-o':
				from .src.channel_analyzer import old
				url_data = swy(path=path)
				print('[*]Start of analysis')
				try:
					min_tps = int(sys.argv[arg + 1])
				except:
					old(url_data)
				else:
					old(url_data, min_tps)
			elif sys.argv[arg] == '-d':
				from .src.channel_analyzer import dead
				url_data = swy(path=path)
				print('[*]Start of analysis')
				dead(url_data)
			elif sys.argv[arg] == '-m':
				analyze = True
				if sys.argv[arg + 1] in ['html', 'raw', 'list']:
					mode = sys.argv[arg + 1]
				else:
					print('[!] Mode file invalid')
					exit()
			elif sys.argv[arg] == '-t':
				analyze = True
				try:
					count = int(sys.argv[arg + 1])
				except:
					print('[!] Numbers of day invalid')
					exit()
			elif sys.argv[arg] == '-n':
				if os.path.exists(sys.argv[arg + 1]):
					if os.path.exists('sub.swy'):
						os.remove('sub.swy')
					generate_swy(sys.argv[arg + 1], path)
				else:
					print('[!] File not found')
					exit()
			elif sys.argv[arg] == '-a':
				if sys.argv[arg + 1][:2] in ['UC', 'PL']:
					add_sub([sys.argv[arg + 1]], path)
				else:
					print('[!] Id is not available')
			elif sys.argv[arg] == '-af':
				if os.path.exists(sys.argv[arg + 1]):
					add_sub(open(sys.argv[arg + 1], 'r').read().list('\n'), path)
				else:
					print('[!] File not found')
					exit()
			elif sys.argv[arg] == '-ax':
				if os.path.exists(sys.argv[arg + 1]):
					generate_swy(sys.argv[arg + 1], path)
				else:
					print('[!] File not found')
					exit()
			elif sys.argv[arg] == '-l':
				analyze = True
				analyze_only_one = True
				if sys.argv[arg + 1][:2] in ['UC', 'PL']:
					url_data = [sys.argv[arg + 1]]
				else:
					print('[!] Id is not available')
			elif sys.argv[arg] == '-r':
				from shutil import rmtree
				if os.path.exists('data'):
					rmtree('data')
				else:
					print('[!] Data do not exist')
			elif sys.argv[arg] == '-s':
				if check_id(sys.argv[arg+1]):
					from .src.channel_analyzer import stat
					stat(sys.argv[arg+1])
				elif sys.argv[arg+1] == 'all':
					from .src.channel_analyzer import stats
					subs = swy(path=path, liste=False)
					stats(subs)
				else:
					print("[!] Id is not available")

	if analyze:
		passe = time.time()
		if not analyze_only_one:
			url_data = swy(sub_file, path=path)
		if mode == 'html':
			html_init()
		elif mode == 'raw':
			if os.path.exists('sub_raw'):
				os.remove('sub_raw')
		elif mode == 'list':
			if os.path.exists('sub_list'):
				os.remove('sub_list')
		nb_new = init(url_data, lcl_time(), path, mode)
		if mode == 'html':
			html_end(count, path)
		elif mode ==  'raw':
			raw_end(count)
		elif mode == 'list':
			list_end(count)
		open(path + 'log', 'a').write(str(time.time() - passe) + '\t' + str(nb_new) + '\t' + time.strftime("    %H%M") + '\n')



import os 
import time
import socket
from threading import Thread

def xml_recup(url):
	nb = 0
	data = b""
	if url[:2] == 'UC':
		url_xml = b'GET /feeds/videos.xml?channel_id=' + url.encode()
	elif url[:2] == 'PL':
		url_xml = b'GET /feeds/videos.xml?playlist_id=' + url.encode()
	else:
		return None
	sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	sock.connect(("youtube.com", 80))
	sock.send(url_xml + b" HTTP/1.0\r\nHost: www.youtube.com\r\n\r\n")
	while True:
		raw_data = sock.recv(1024)
		if raw_data == b"":
			break
		else:
			data += raw_data
	sock.close()
	data = data.decode('utf8')
	linfo = data.split("<entry>")
	del linfo[0]
	if linfo == []:
		return None
	return linfo

def html_init(path):
	open('sub.html', 'w').write("""<html>
	<head>
		<meta charset="utf-8" />
		<link rel="stylesheet" href="css/sub.css" />
		<link rel="stylesheet" media="screen and (max-width: 1081px)" href="css/sub_mobile.css"/>
		<title>Abonnements</title>
	</head>
	<body>
<!-- {} -->
""".format(time.ctime()))

def init(urls, min_date, path='', mode='html'):
	threads = []
	for url in urls:
		thr = Analyzer(mode, url, min_date, path)
		thr.Thread()
		threads.append(thr)
		thr.start()
	for i in threads:
		i.join()

class Analyzer(Thread):
	def __init__(self, mode='', url='', min_date='', path=''):
		self.mode = mode
		self.url = url
		self.path_cache = path
		self.min_date = min_date
		#all the var user in class
		self.url_channel = ""
		self.title = ""
		self.channel = ""
		self.date = []
		self.image = ""
	
	def Thread(self):
		Thread.__init__(self)
	
	def run(self):
		self.analyzer_sub()

	def analyzer_sub(self):
		linfo = xml_recup(self.url)
		nb_new = 0
		if linfo == False or linfo == None:
			return 0
		for i in linfo:
			date = int(i.split("<published>")[1].split("</published>")[0].replace('-', '').replace('+00:00', '').replace('T', '').replace(':', ''))
			if self.min_date <= date:
				dvid = self.info_recup(i)
				if dvid:
					nb_new += 1
				else:
					return 0

	def info_recup(self, i):
		self.url = i.split('<yt:videoId>')[1].split('</yt:videoId>')[0]
		self.url_channel = i.split('<yt:channelId>')[1].split('</yt:channelId>')[0]
		self.title = i.split('<media:title>')[1].split('</media:title>')[0]
		self.channel = i.split('<name>')[1].split('</name>')[0]
		self.date = i.split('<published>')[1].split('+')[0].split('T')
		self.image = 'https://i.ytimg.com/vi/' + self.url  + '/mqdefault.jpg'
		if self.mode == 'html':
			return self.generate_data_html()
		elif self.mode == 'raw':
			return self.append_raw()
		elif self.mode == 'list':
			return self.append_list()

	def append_raw(self):
		open('sub_raw', 'a', encoding='utf8').write(self.date[0] + '\t' + self.url + '\t' + self.url_channel + '\t' + self.title + '\t' + self.channel + '\t' + self.image + '\n')
		return True

	def append_list(self):
		open('sub_list', 'a', encoding='utf8').write(self.date[0] + ' https://www.youtube.com/watch?v=' + self.url + '\n')
		return True

	def generate_data_html(self):
		try:
			data = open(self.path_cache + 'data/' + self.date[0] + '/' + self.date[1].replace(':', ''), 'rb+').read().decode("utf8")
			if self.url in data:
				return False
		except:
			try:
				os.mkdir(self.path_cache + 'data/' + self.date[0])
			except:
				pass
		open(self.path_cache + 'data/' + self.date[0] + '/' + self.date[1].replace(':', ''), 'a', encoding='utf-8').write("""<!--NEXT -->
	<div class="video">
		<a class="left" href="https://www.youtube.com/watch?v={}"> <img src="{}" ></a>
		<a href="https://www.youtube.com/watch?v={}"><h4>{}</h4> </a>
		<a href="https://www.youtube.com/channel/{}"> <p>{}</p> </a>
		<p>{}</p>
		<p class="clear"></p>
	</div>
	""".format(self.url, self.image, self.url, self.title, self.url_channel, self.channel, self.date[0]))
		return True

def html_end(count=7, path=''):
	fch = sorted(os.listdir(path + 'data/'))
	if len(fch) < count:
		count = len(fch)
	elif count == -1:
		count = len(fch)
	for i in range(count):
		fch_in = sorted(os.listdir(path + 'data/' + fch[-1-i]))
		for a in range(len(fch_in)):
			data = open(path + 'data/' + fch[-1-i] + '/' + fch_in[-1-a], 'r', encoding='utf-8').read()
			open('sub.html', 'a', encoding='utf-8').write(data)
	open('sub.html', 'a').write('</body></html>')

def raw_end(count=7):
	nb = 0
	linfo = sorted(open('sub_raw', 'r').read().split('\n'))
	for i in range(len(linfo)):
		if linfo[-1-i][:10] != linfo[-2-i][:10]:
			nb += 1
			if nb == count:
				nb = i
				break
	os.remove('sub_raw')
	fichier = open('sub_raw', 'a', encoding='utf8')
	for i in range(nb):
		fichier.write(linfo[-1-i] + '\n')

def list_end(count=7):
	nb = 0
	linfo = sorted(open('sub_list', 'r').read().split('\n'))
	if count == -1:
		nb = len(linfo)
	else:
		for i in range(len(linfo)):
			if linfo[-1-i][:10] != linfo[-2-i][:10]:
				nb += 1
				if nb == count:
					nb = i
					break
	os.remove('sub_list')
	fichier = open('sub_list', 'a', encoding='utf8')
	for i in range(nb):
		fichier.write(linfo[-1-i][11:] + '\n')

